template <typename T>
void call_function_feed_argument(T& function, jl_value_t* value)
{
    jl_value_t* current_type = jl_typeof(value);
    if (jl_is_int64(value)) {
        function << static_cast<long>(jl_unbox_int64(value));
    } else if (jl_is_bool(value)) {
        function << jl_unbox_bool(value);
    } else if (jl_is_string(value)) {
        function << std::string(jl_string_data(value));
    } else if (jl_typeis(value, jl_float64_type)){
        function << jl_unbox_float64(value);
    }     else if (jl_subtype(current_type, POLYMAKETYPE_PropertyValue)) {
        function << jlcxx::unbox<const pm::perl::PropertyValue&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_OptionSet)) {
        function << jlcxx::unbox<const pm::perl::OptionSet&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_BigObject)) {
        function << jlcxx::unbox<const pm::perl::BigObject&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Integer)) {
        function << jlcxx::unbox<const pm::Integer&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Rational)) {
        function << jlcxx::unbox<const pm::Rational&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_QuadraticExtension_Rational)) {
        function << jlcxx::unbox<const pm::QuadraticExtension<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Polynomial_Rational_Int)) {
        function << jlcxx::unbox<const pm::Matrix<pm::Polynomial<pm::Rational,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Polynomial_Rational_Int)) {
        function << jlcxx::unbox<const pm::Vector<pm::Polynomial<pm::Rational,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Int_Int)) {
        function << jlcxx::unbox<const std::pair<pm::Int, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Array_Int_Array_Int)) {
        function << jlcxx::unbox<const std::pair<pm::Array<pm::Int>, pm::Array<pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Pair_Array_Int_Array_Int)) {
        function << jlcxx::unbox<const pm::Array<std::pair<pm::Array<pm::Int>, pm::Array<pm::Int>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Int_List_List_Pair_Int_Int)) {
        function << jlcxx::unbox<const std::pair<pm::Int, std::list<std::list<std::pair<pm::Int, pm::Int>>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Integer_Int)) {
        function << jlcxx::unbox<const std::pair<pm::Integer, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_List_Pair_Int_Int)) {
        function << jlcxx::unbox<const std::list<std::pair<pm::Int, pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_List_List_Pair_Int_Int)) {
        function << jlcxx::unbox<const std::list<std::list<std::pair<pm::Int, pm::Int>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_List_Pair_Integer_Int)) {
        function << jlcxx::unbox<const std::list<std::pair<pm::Integer, pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Set_Int)) {
        function << jlcxx::unbox<const pm::Set<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_String)) {
        function << jlcxx::unbox<const pm::Array<std::string>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Set_Int)) {
        function << jlcxx::unbox<const pm::Array<pm::Set<long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Int)) {
        function << jlcxx::unbox<const pm::Array<pm::Array<long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Integer)) {
        function << jlcxx::unbox<const pm::Array<pm::Array<pm::Integer>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Set_Int)) {
        function << jlcxx::unbox<const pm::Array<pm::Array<pm::Set<long>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Rational)) {
        function << jlcxx::unbox<const pm::Array<pm::Array<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Pair_Int_Int)) {
        function << jlcxx::unbox<const pm::Array<std::pair<pm::Int, pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_List_Pair_Int_Int)) {
        function << jlcxx::unbox<const pm::Array<std::list<std::pair<pm::Int, pm::Int>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Matrix_Integer)) {
        function << jlcxx::unbox<const pm::Array<pm::Matrix<pm::Integer>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_HomologyGroup_Integer)) {
        function << jlcxx::unbox<const pm::Array<polymake::topaz::HomologyGroup<pm::Integer>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_BigObject)) {
        function << jlcxx::unbox<const pm::Array<pm::perl::BigObject>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Polynomial_Integer_Int)) {
        function << jlcxx::unbox<const pm::Array<pm::Polynomial<pm::Integer,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Polynomial_Rational_Int)) {
        function << jlcxx::unbox<const pm::Array<pm::Polynomial<pm::Rational,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_IncidenceMatrix_NonSymmetric)) {
        function << jlcxx::unbox<const pm::IncidenceMatrix<pm::NonSymmetric>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_IncidenceMatrix_Symmetric)) {
        function << jlcxx::unbox<const pm::IncidenceMatrix<pm::Symmetric>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_TropicalNumber_Max_Rational)) {
        function << jlcxx::unbox<const pm::TropicalNumber<pm::Max,pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_TropicalNumber_Min_Rational)) {
        function << jlcxx::unbox<const pm::TropicalNumber<pm::Min,pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_Int_Int)) {
        function << jlcxx::unbox<const pm::UniPolynomial<long,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_Integer_Int)) {
        function << jlcxx::unbox<const pm::UniPolynomial<pm::Integer,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_Rational_Int)) {
        function << jlcxx::unbox<const pm::UniPolynomial<pm::Rational,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_QuadraticExtension_Rational_Int)) {
        function << jlcxx::unbox<const pm::UniPolynomial<pm::QuadraticExtension<pm::Rational>,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_Int_Int)) {
        function << jlcxx::unbox<const pm::Polynomial<long,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_Integer_Int)) {
        function << jlcxx::unbox<const pm::Polynomial<pm::Integer,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_Rational_Int)) {
        function << jlcxx::unbox<const pm::Polynomial<pm::Rational,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_double_Int)) {
        function << jlcxx::unbox<const pm::Polynomial<double,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_QuadraticExtension_Rational_Int)) {
        function << jlcxx::unbox<const pm::Polynomial<pm::QuadraticExtension<pm::Rational>,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Map_String_String)) {
        function << jlcxx::unbox<const pm::Map<std::string,std::string>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_HomologyGroup_Integer)) {
        function << jlcxx::unbox<const polymake::topaz::HomologyGroup<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Graph_Undirected)) {
        function << jlcxx::unbox<const pm::graph::Graph<pm::graph::Undirected>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Graph_Directed)) {
        function << jlcxx::unbox<const pm::graph::Graph<pm::graph::Directed>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_EdgeMap_Directed_Int)) {
        function << jlcxx::unbox<const pm::graph::EdgeMap<pm::graph::Directed, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_EdgeMap_Undirected_Int)) {
        function << jlcxx::unbox<const pm::graph::EdgeMap<pm::graph::Undirected, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Directed_Int)) {
        function << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Directed, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Undirected_Int)) {
        function << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Undirected, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Directed_Set_Int)) {
        function << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Directed, pm::Set<pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Undirected_Set_Int)) {
        function << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Undirected, pm::Set<pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Int)) {
        function << jlcxx::unbox<const pm::Matrix<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Integer)) {
        function << jlcxx::unbox<const pm::Matrix<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Rational)) {
        function << jlcxx::unbox<const pm::Matrix<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_double)) {
        function << jlcxx::unbox<const pm::Matrix<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_QuadraticExtension_Rational)) {
        function << jlcxx::unbox<const pm::Matrix<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_Int)) {
        function << jlcxx::unbox<const pm::SparseMatrix<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_Integer)) {
        function << jlcxx::unbox<const pm::SparseMatrix<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_Rational)) {
        function << jlcxx::unbox<const pm::SparseMatrix<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_double)) {
        function << jlcxx::unbox<const pm::SparseMatrix<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_QuadraticExtension_Rational)) {
        function << jlcxx::unbox<const pm::SparseMatrix<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Int)) {
        function << jlcxx::unbox<const pm::Vector<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Integer)) {
        function << jlcxx::unbox<const pm::Vector<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Rational)) {
        function << jlcxx::unbox<const pm::Vector<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_double)) {
        function << jlcxx::unbox<const pm::Vector<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_QuadraticExtension_Rational)) {
        function << jlcxx::unbox<const pm::Vector<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_Int)) {
        function << jlcxx::unbox<const pm::SparseVector<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_Integer)) {
        function << jlcxx::unbox<const pm::SparseVector<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_Rational)) {
        function << jlcxx::unbox<const pm::SparseVector<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_double)) {
        function << jlcxx::unbox<const pm::SparseVector<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_QuadraticExtension_Rational)) {
        function << jlcxx::unbox<const pm::SparseVector<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Int)) {
        function << jlcxx::unbox<const pm::Array<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Integer)) {
        function << jlcxx::unbox<const pm::Array<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Rational)) {
        function << jlcxx::unbox<const pm::Array<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_double)) {
        function << jlcxx::unbox<const pm::Array<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_QuadraticExtension_Rational)) {
        function << jlcxx::unbox<const pm::Array<pm::QuadraticExtension<pm::Rational>>&>(value);
    }

    else {
        for (const auto& feeder : feeder_list<T&>::get()) {
            if (feeder(function, value))
                return;
        }
        throw std::runtime_error(
            "Cannot pass function value: conversion failed for argument of type " + std::string(jl_typeof_str(value))
        );
    }
    return;
}

