void option_set_take(pm::perl::OptionSet optset,
                     std::string         key,
                     jl_value_t*         value)
{
    jl_value_t* current_type = jl_typeof(value);
    if (jl_is_int64(value)) {
        optset[key] << static_cast<long>(jl_unbox_int64(value));
    } else if (jl_is_bool(value)) {
        optset[key] << jl_unbox_bool(value);
    } else if (jl_is_string(value)) {
        optset[key] << std::string(jl_string_data(value));
    } else if (jl_typeis(value, jl_float64_type)){
        optset[key] << jl_unbox_float64(value);
    }     else if (jl_subtype(current_type, POLYMAKETYPE_PropertyValue)) {
        optset[key] << jlcxx::unbox<const pm::perl::PropertyValue&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_OptionSet)) {
        optset[key] << jlcxx::unbox<const pm::perl::OptionSet&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_BigObject)) {
        optset[key] << jlcxx::unbox<const pm::perl::BigObject&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Integer)) {
        optset[key] << jlcxx::unbox<const pm::Integer&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Rational&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_QuadraticExtension_Rational)) {
        optset[key] << jlcxx::unbox<const pm::QuadraticExtension<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Polynomial_Rational_Int)) {
        optset[key] << jlcxx::unbox<const pm::Matrix<pm::Polynomial<pm::Rational,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Polynomial_Rational_Int)) {
        optset[key] << jlcxx::unbox<const pm::Vector<pm::Polynomial<pm::Rational,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Int_Int)) {
        optset[key] << jlcxx::unbox<const std::pair<pm::Int, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Array_Int_Array_Int)) {
        optset[key] << jlcxx::unbox<const std::pair<pm::Array<pm::Int>, pm::Array<pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Pair_Array_Int_Array_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<std::pair<pm::Array<pm::Int>, pm::Array<pm::Int>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Int_List_List_Pair_Int_Int)) {
        optset[key] << jlcxx::unbox<const std::pair<pm::Int, std::list<std::list<std::pair<pm::Int, pm::Int>>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Pair_Integer_Int)) {
        optset[key] << jlcxx::unbox<const std::pair<pm::Integer, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_List_Pair_Int_Int)) {
        optset[key] << jlcxx::unbox<const std::list<std::pair<pm::Int, pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_List_List_Pair_Int_Int)) {
        optset[key] << jlcxx::unbox<const std::list<std::list<std::pair<pm::Int, pm::Int>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_List_Pair_Integer_Int)) {
        optset[key] << jlcxx::unbox<const std::list<std::pair<pm::Integer, pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Set_Int)) {
        optset[key] << jlcxx::unbox<const pm::Set<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_String)) {
        optset[key] << jlcxx::unbox<const pm::Array<std::string>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Set_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Set<long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Array<long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Integer)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Array<pm::Integer>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Set_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Array<pm::Set<long>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Array_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Array<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Pair_Int_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<std::pair<pm::Int, pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_List_Pair_Int_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<std::list<std::pair<pm::Int, pm::Int>>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Matrix_Integer)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Matrix<pm::Integer>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_HomologyGroup_Integer)) {
        optset[key] << jlcxx::unbox<const pm::Array<polymake::topaz::HomologyGroup<pm::Integer>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_BigObject)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::perl::BigObject>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Polynomial_Integer_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Polynomial<pm::Integer,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Polynomial_Rational_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Polynomial<pm::Rational,long>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_IncidenceMatrix_NonSymmetric)) {
        optset[key] << jlcxx::unbox<const pm::IncidenceMatrix<pm::NonSymmetric>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_IncidenceMatrix_Symmetric)) {
        optset[key] << jlcxx::unbox<const pm::IncidenceMatrix<pm::Symmetric>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_TropicalNumber_Max_Rational)) {
        optset[key] << jlcxx::unbox<const pm::TropicalNumber<pm::Max,pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_TropicalNumber_Min_Rational)) {
        optset[key] << jlcxx::unbox<const pm::TropicalNumber<pm::Min,pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_Int_Int)) {
        optset[key] << jlcxx::unbox<const pm::UniPolynomial<long,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_Integer_Int)) {
        optset[key] << jlcxx::unbox<const pm::UniPolynomial<pm::Integer,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_Rational_Int)) {
        optset[key] << jlcxx::unbox<const pm::UniPolynomial<pm::Rational,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_UniPolynomial_QuadraticExtension_Rational_Int)) {
        optset[key] << jlcxx::unbox<const pm::UniPolynomial<pm::QuadraticExtension<pm::Rational>,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_Int_Int)) {
        optset[key] << jlcxx::unbox<const pm::Polynomial<long,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_Integer_Int)) {
        optset[key] << jlcxx::unbox<const pm::Polynomial<pm::Integer,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_Rational_Int)) {
        optset[key] << jlcxx::unbox<const pm::Polynomial<pm::Rational,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_double_Int)) {
        optset[key] << jlcxx::unbox<const pm::Polynomial<double,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Polynomial_QuadraticExtension_Rational_Int)) {
        optset[key] << jlcxx::unbox<const pm::Polynomial<pm::QuadraticExtension<pm::Rational>,long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Map_String_String)) {
        optset[key] << jlcxx::unbox<const pm::Map<std::string,std::string>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_HomologyGroup_Integer)) {
        optset[key] << jlcxx::unbox<const polymake::topaz::HomologyGroup<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Graph_Undirected)) {
        optset[key] << jlcxx::unbox<const pm::graph::Graph<pm::graph::Undirected>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Graph_Directed)) {
        optset[key] << jlcxx::unbox<const pm::graph::Graph<pm::graph::Directed>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_EdgeMap_Directed_Int)) {
        optset[key] << jlcxx::unbox<const pm::graph::EdgeMap<pm::graph::Directed, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_EdgeMap_Undirected_Int)) {
        optset[key] << jlcxx::unbox<const pm::graph::EdgeMap<pm::graph::Undirected, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Directed_Int)) {
        optset[key] << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Directed, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Undirected_Int)) {
        optset[key] << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Undirected, pm::Int>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Directed_Set_Int)) {
        optset[key] << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Directed, pm::Set<pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_NodeMap_Undirected_Set_Int)) {
        optset[key] << jlcxx::unbox<const pm::graph::NodeMap<pm::graph::Undirected, pm::Set<pm::Int>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Int)) {
        optset[key] << jlcxx::unbox<const pm::Matrix<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Integer)) {
        optset[key] << jlcxx::unbox<const pm::Matrix<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Matrix<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_double)) {
        optset[key] << jlcxx::unbox<const pm::Matrix<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Matrix_QuadraticExtension_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Matrix<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_Int)) {
        optset[key] << jlcxx::unbox<const pm::SparseMatrix<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_Integer)) {
        optset[key] << jlcxx::unbox<const pm::SparseMatrix<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_Rational)) {
        optset[key] << jlcxx::unbox<const pm::SparseMatrix<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_double)) {
        optset[key] << jlcxx::unbox<const pm::SparseMatrix<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseMatrix_QuadraticExtension_Rational)) {
        optset[key] << jlcxx::unbox<const pm::SparseMatrix<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Int)) {
        optset[key] << jlcxx::unbox<const pm::Vector<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Integer)) {
        optset[key] << jlcxx::unbox<const pm::Vector<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Vector<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_double)) {
        optset[key] << jlcxx::unbox<const pm::Vector<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Vector_QuadraticExtension_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Vector<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_Int)) {
        optset[key] << jlcxx::unbox<const pm::SparseVector<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_Integer)) {
        optset[key] << jlcxx::unbox<const pm::SparseVector<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_Rational)) {
        optset[key] << jlcxx::unbox<const pm::SparseVector<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_double)) {
        optset[key] << jlcxx::unbox<const pm::SparseVector<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_SparseVector_QuadraticExtension_Rational)) {
        optset[key] << jlcxx::unbox<const pm::SparseVector<pm::QuadraticExtension<pm::Rational>>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Int)) {
        optset[key] << jlcxx::unbox<const pm::Array<long>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Integer)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Integer>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::Rational>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_double)) {
        optset[key] << jlcxx::unbox<const pm::Array<double>&>(value);
    }
    else if (jl_subtype(current_type, POLYMAKETYPE_Array_QuadraticExtension_Rational)) {
        optset[key] << jlcxx::unbox<const pm::Array<pm::QuadraticExtension<pm::Rational>>&>(value);
    }

    else {
        for (const auto& feeder : feeder_list<pm::perl::Value>::get()) {
            if (feeder(optset[key], value))
                return;
        }
        throw std::runtime_error(
            "Cannot create OptionSet: conversion failed for (key, value) = (" +
            key +
            ", ::" +
            std::string(jl_typeof_str(value)) +
            ")"
        );
    }
    return;
}

